@@ -42,79 +42,44 @@
 
 #include "kmime_util.h"
 #include "kmime_util_p.h"
+#include "kmime_codecs.h"
 #include "kmime_content.h"
 #include "kmime_header_parsing.h"
 #include "kmime_headerfactory_p.h"
 #include "kmime_warning.h"
 
 
+#include <kcharsets.h>
+#include <KCodecs>
+
 #include <QtCore/QTextCodec>
 #include <QtCore/QString>
 #include <QtCore/QStringList>
 
-#include <kcharsets.h>
-
 #include <assert.h>
 #include <ctype.h>
 
-template <typename T>
-bool registerHeaderHelper()
-{
-    const T dummy;
-    if (QByteArray(dummy.type()).isEmpty()) {
-        // This is a generic header.
-        return false;
-    }
-    return KMime::HeaderFactory::self()->registerHeader<T>();
-}
-
-// macro to register a header with HeaderFactory
-#define kmime_register_header( subclass )                             \
-    namespace { const bool dummyForRegistering##subclass = registerHeaderHelper<subclass>(); }
-
 // macro to generate a default constructor implementation
 #define kmime_mk_trivial_ctor( subclass, baseclass )                  \
-    subclass::subclass( Content *parent ) : baseclass( parent )           \
+    subclass::subclass() : baseclass()           \
     {                                                                     \
-        clear();                                                            \
     }                                                                     \
     \
-    subclass::subclass( Content *parent, const QByteArray &s ) : baseclass( parent ) \
-    {                                                                     \
-        from7BitString( s );                                                \
-    }                                                                     \
-    \
-    subclass::subclass( Content *parent, const QString &s, const QByteArray &charset ) : \
-        baseclass( parent )                                                 \
-    {                                                                     \
-        fromUnicodeString( s, charset );                                    \
-    }                                                                     \
-    \
-    subclass::~subclass() {}                                              \
-    \
-    kmime_register_header( subclass )
+    subclass::~subclass() {}
+
 // end kmime_mk_trivial_ctor
 
 #define kmime_mk_trivial_ctor_with_dptr( subclass, baseclass ) \
-    subclass::subclass( Content *parent ) : baseclass( new subclass##Private, parent ) \
+    subclass::subclass() : baseclass( new subclass##Private ) \
     {                                                                     \
-        clear();                                                            \
     }                                                                     \
     \
-    subclass::subclass( Content *parent, const QByteArray &s ) : baseclass( new subclass##Private, parent ) \
-    {                                                                     \
-        from7BitString( s );                                                \
-    }                                                                     \
-    \
-    subclass::subclass( Content *parent, const QString &s, const QByteArray &charset ) : \
-        baseclass( new subclass##Private, parent )                          \
-    {                                                                     \
-        fromUnicodeString( s, charset );                                    \
-    }                                                                     \
-    \
-    subclass::~subclass() {}                                              \
-    \
-    kmime_register_header( subclass )
+    subclass::~subclass() { \
+        Q_D(subclass); \
+        delete d; \
+        d_ptr = 0; \
+    }
+
 // end kmime_mk_trivial_ctor_with_dptr
 
 #define kmime_mk_trivial_ctor_with_name( subclass, baseclass, name )  \
@@ -132,7 +97,7 @@ bool registerHeaderHelper()
     const char *subclass::staticType() { return #name; }
 
 #define kmime_mk_dptr_ctor( subclass, baseclass ) \
-    subclass::subclass( subclass##Private *d, KMime::Content *parent ) : baseclass( d, parent ) {}
+    subclass::subclass( subclass##Private *d ) : baseclass( d ) {}
 
 using namespace KMime;
 using namespace KMime::Headers;
@@ -144,18 +109,13 @@ namespace KMime
 namespace Headers
 {
 //-----<Base>----------------------------------
-Base::Base(KMime::Content *parent) :
-    d_ptr(new BasePrivate)
+Base::Base() : d_ptr(new BasePrivate)
 {
-    Q_D(Base);
-    d->parent = parent;
 }
 
-Base::Base(BasePrivate *dd, KMime::Content *parent) :
+Base::Base(BasePrivate *dd) :
     d_ptr(dd)
 {
-    Q_D(Base);
-    d->parent = parent;
 }
 
 Base::~Base()
@@ -164,20 +124,10 @@ Base::~Base()
     d_ptr = 0;
 }
 
-KMime::Content *Base::parent() const
-{
-    return d_ptr->parent;
-}
-
-void Base::setParent(KMime::Content *parent)
-{
-    d_ptr->parent = parent;
-}
-
 QByteArray Base::rfc2047Charset() const
 {
-    if (d_ptr->encCS.isEmpty() || forceDefaultCharset()) {
-        return defaultCharset();
+    if (d_ptr->encCS.isEmpty()) {
+        return Content::defaultCharset();
     } else {
         return d_ptr->encCS;
     }
@@ -188,16 +138,6 @@ void Base::setRFC2047Charset(const QByteArray &cs)
     d_ptr->encCS = cachedCharset(cs);
 }
 
-bool Base::forceDefaultCharset() const
-{
-    return (parent() != 0 ? parent()->forceDefaultCharset() : false);
-}
-
-QByteArray Base::defaultCharset() const
-{
-    return (parent() != 0 ? parent()->defaultCharset() : "ISO-8859-1");
-}
-
 const char *Base::type() const
 {
     return "";
@@ -213,11 +153,6 @@ bool Base::isMimeHeader() const
     return qstrnicmp(type(), "Content-", 8) == 0;
 }
 
-bool Base::isXHeader() const
-{
-    return qstrncmp(type(), "X-", 2) == 0;
-}
-
 QByteArray Base::typeIntro() const
 {
     return QByteArray(type()) + ": ";
@@ -234,28 +169,21 @@ namespace Generics
 kmime_mk_dptr_ctor(Unstructured, Base)
 //@endcond
 
-Unstructured::Unstructured(Content *p) : Base(new UnstructuredPrivate, p)
+Unstructured::Unstructured() : Base(new UnstructuredPrivate)
 {
 }
 
-Unstructured::Unstructured(Content *p, const QByteArray &s) : Base(new UnstructuredPrivate, p)
-{
-    from7BitString(s);
-}
-
-Unstructured::Unstructured(Content *p, const QString &s, const QByteArray &cs) : Base(new UnstructuredPrivate, p)
-{
-    fromUnicodeString(s, cs);
-}
-
 Unstructured::~Unstructured()
 {
+    Q_D(Unstructured);
+    delete d;
+    d_ptr = 0;
 }
 
 void Unstructured::from7BitString(const QByteArray &s)
 {
     Q_D(Unstructured);
-    d->decoded = decodeRFC2047String(s, d->encCS, defaultCharset(), forceDefaultCharset());
+    d->decoded = KCodecs::decodeRFC2047String(s, &d->encCS, Content::defaultCharset());
 }
 
 QByteArray Unstructured::as7BitString(bool withHeaderType) const
@@ -297,7 +225,7 @@ bool Unstructured::isEmpty() const
 
 //-----<Structured>-------------------------
 
-Structured::Structured(Content *p) : Base(new StructuredPrivate, p)
+Structured::Structured() : Base(new StructuredPrivate)
 {
 }
 
@@ -305,13 +233,16 @@ kmime_mk_dptr_ctor(Structured, Base)
 
 Structured::~Structured()
 {
+    Q_D(Structured);
+    delete d;
+    d_ptr = 0;
 }
 
 void Structured::from7BitString(const QByteArray &s)
 {
     Q_D(Structured);
     if (d->encCS.isEmpty()) {
-        d->encCS = defaultCharset();
+        d->encCS = Content::defaultCharset();
     }
     const char *cursor = s.constData();
     parse(cursor, cursor + s.length());
@@ -333,20 +264,10 @@ void Structured::fromUnicodeString(const QString &s, const QByteArray &b)
 
 //-----<Address>-------------------------
 
-Address::Address(Content *p) : Structured(new AddressPrivate, p)
+Address::Address() : Structured(new AddressPrivate)
 {
 }
 
-Address::Address(Content *p, const QByteArray &s) : Structured(new AddressPrivate, p)
-{
-    from7BitString(s);
-}
-
-Address::Address(Content *p, const QString &s, const QByteArray &cs) : Structured(new AddressPrivate, p)
-{
-    fromUnicodeString(s, cs);
-}
-
 kmime_mk_dptr_ctor(Address, Structured)
 
 Address:: ~Address()
@@ -405,7 +326,8 @@ void MailboxList::fromUnicodeString(const QString &s, const QByteArray &b)
 
 QString MailboxList::asUnicodeString() const
 {
-    return prettyAddresses().join(QLatin1String(", "));
+    Q_D(const MailboxList);
+    return Mailbox::listToUnicodeString(d->mailboxList);
 }
 
 void MailboxList::clear()
@@ -435,9 +357,9 @@ void MailboxList::addAddress(const QByteArray &address,
     }
 }
 
-QList< QByteArray > MailboxList::addresses() const
+QVector<QByteArray> MailboxList::addresses() const
 {
-    QList<QByteArray> rv;
+    QVector<QByteArray> rv;
     rv.reserve(d_func()->mailboxList.count());
     foreach (const Types::Mailbox &mbox, d_func()->mailboxList) {
         rv.append(mbox.address());
@@ -447,22 +369,29 @@ QList< QByteArray > MailboxList::addresses() const
 
 QStringList MailboxList::displayNames() const
 {
+    Q_D(const MailboxList);
     QStringList rv;
-    rv.reserve(d_func()->mailboxList.count());
-    foreach (const Types::Mailbox &mbox, d_func()->mailboxList) {
-        rv.append(mbox.name());
+    rv.reserve(d->mailboxList.count());
+    foreach (const Types::Mailbox &mbox, d->mailboxList) {
+        if (mbox.hasName())
+            rv.append(mbox.name());
+        else
+            rv.append(QString::fromLatin1(mbox.address()));
     }
     return rv;
 }
 
-QStringList MailboxList::prettyAddresses() const
+QString MailboxList::displayString() const
 {
-    QStringList rv;
-    rv.reserve(d_func()->mailboxList.count());
-    foreach (const Types::Mailbox &mbox, d_func()->mailboxList) {
-        rv.append(mbox.prettyAddress());
+    Q_D(const MailboxList);
+    if (d->mailboxList.size() == 1) { // fast-path to avoid temporary QStringList in the common case of just one From address
+        const auto& mbox = d->mailboxList.at(0);
+        if (mbox.hasName())
+            return mbox.name();
+        else
+            return QString::fromLatin1(mbox.address());
     }
-    return rv;
+    return displayNames().join(QStringLiteral(", "));
 }
 
 Types::Mailbox::List MailboxList::mailboxes() const
@@ -488,7 +417,7 @@ bool MailboxList::parse(const char *&scursor, const char *const send,
     d->mailboxList.reserve(maybeAddressList.count());
 
     // extract the mailboxes and complain if there are groups:
-    foreach ( auto it, maybeAddressList) {
+    foreach (const auto &it, maybeAddressList) {
         if (!(it).displayName.isEmpty()) {
             KMIME_WARN << "mailbox groups in header disallowing them! Name: \""
                        << (it).displayName << "\"" << endl;
@@ -560,7 +489,15 @@ void AddressList::fromUnicodeString(const QString &s, const QByteArray &b)
 
 QString AddressList::asUnicodeString() const
 {
-    return prettyAddresses().join(QLatin1String(", "));
+    Q_D(const AddressList);
+    QStringList rv;
+    foreach (const Types::Address &addr, d->addressList) {
+        rv.reserve(rv.size() + addr.mailboxList.size());
+        foreach (const Types::Mailbox &mbox, addr.mailboxList) {
+            rv.append(mbox.prettyAddress());
+        }
+    }
+    return rv.join(QStringLiteral(", "));
 }
 
 void AddressList::clear()
@@ -594,9 +531,9 @@ void AddressList::addAddress(const QByteArray &address,
     }
 }
 
-QList< QByteArray > AddressList::addresses() const
+QVector<QByteArray> AddressList::addresses() const
 {
-    QList<QByteArray> rv;
+    QVector<QByteArray> rv;
     foreach (const Types::Address &addr, d_func()->addressList) {
         foreach (const Types::Mailbox &mbox, addr.mailboxList) {
             rv.append(mbox.address());
@@ -607,24 +544,23 @@ QList< QByteArray > AddressList::addresses() const
 
 QStringList AddressList::displayNames() const
 {
+    Q_D(const AddressList);
     QStringList rv;
-    foreach (const Types::Address &addr, d_func()->addressList) {
+    foreach (const Types::Address &addr, d->addressList) {
         foreach (const Types::Mailbox &mbox, addr.mailboxList) {
-            rv.append(mbox.name());
+            if (mbox.hasName())
+                rv.append(mbox.name());
+            else
+                rv.append(QString::fromLatin1(mbox.address()));
         }
     }
     return rv;
 }
 
-QStringList AddressList::prettyAddresses() const
+QString AddressList::displayString() const
 {
-    QStringList rv;
-    foreach (const Types::Address &addr, d_func()->addressList) {
-        foreach (const Types::Mailbox &mbox, addr.mailboxList) {
-            rv.append(mbox.prettyAddress());
-        }
-    }
-    return rv;
+    // optimize for single entry and avoid creation of the QStringList in that case?
+    return displayNames().join(QStringLiteral(", "));
 }
 
 Types::Mailbox::List AddressList::mailboxes() const
@@ -751,7 +687,7 @@ QByteArray PhraseList::as7BitString(bool withHeaderType) const
 
 QString PhraseList::asUnicodeString() const
 {
-    return d_func()->phraseList.join(QLatin1String(", "));
+    return d_func()->phraseList.join(QStringLiteral(", "));
 }
 
 void PhraseList::clear()
@@ -1046,9 +982,9 @@ bool Ident::parse(const char *&scursor, const char *const send, bool isCRLF)
     return true;
 }
 
-QList<QByteArray> Ident::identifiers() const
+QVector<QByteArray> Ident::identifiers() const
 {
-    QList<QByteArray> rv;
+    QVector<QByteArray> rv;
     foreach (const Types::AddrSpec &addr, d_func()->msgIdList) {
         if (!addr.isEmpty()) {
             const QString asString = addr.asString();
@@ -1229,28 +1165,11 @@ Generic::Generic(const char *t) : Generics::Unstructured(new GenericPrivate)
     setType(t);
 }
 
-Generic::Generic(const char *t, Content *p)
-    : Generics::Unstructured(new GenericPrivate, p)
-{
-    setType(t);
-}
-
-Generic::Generic(const char *t, Content *p, const QByteArray &s)
-    : Generics::Unstructured(new GenericPrivate, p)
-{
-    from7BitString(s);
-    setType(t);
-}
-
-Generic::Generic(const char *t, Content *p, const QString &s, const QByteArray &cs)
-    : Generics::Unstructured(new GenericPrivate, p)
-{
-    fromUnicodeString(s, cs);
-    setType(t);
-}
-
 Generic::~Generic()
 {
+    Q_D(Generic);
+    delete d;
+    d_ptr = 0;
 }
 
 void Generic::clear()
@@ -1410,10 +1329,10 @@ QString MailCopiesTo::asUnicodeString() const
         return AddressList::asUnicodeString();
     }
     if (d_func()->alwaysCopy) {
-        return QLatin1String("poster");
+        return QStringLiteral("poster");
     }
     if (d_func()->neverCopy) {
-        return QLatin1String("nobody");
+        return QStringLiteral("nobody");
     }
     return QString();
 }
@@ -1582,11 +1501,11 @@ bool Newsgroups::isEmpty() const {
     return d_func()->groups.isEmpty();
 }
 
-QList<QByteArray> Newsgroups::groups() const {
+QVector<QByteArray> Newsgroups::groups() const {
     return d_func()->groups;
 }
 
-void Newsgroups::setGroups(const QList<QByteArray> &groups) {
+void Newsgroups::setGroups(const QVector<QByteArray> &groups) {
     Q_D(Newsgroups);
     d->groups = groups;
 }
@@ -1770,6 +1689,12 @@ bool ContentType::isSubtype(const char *subtype) const {
             d->mimeType.size() == pos + len + 1;
 }
 
+bool ContentType::isMimeType(const char* mimeType) const
+{
+    Q_D(const ContentType);
+    return qstricmp(d->mimeType.constData(), mimeType) == 0;
+}
+
 bool ContentType::isText() const {
     return (isMediatype("text") || isEmpty());
 }
@@ -1795,46 +1720,46 @@ bool ContentType::isPartial() const {
 }
 
 QByteArray ContentType::charset() const {
-    QByteArray ret = parameter(QLatin1String("charset")).toLatin1();
-    if (ret.isEmpty() || forceDefaultCharset()) {
+    QByteArray ret = parameter(QStringLiteral("charset")).toLatin1();
+    if (ret.isEmpty()) {
         //return the default-charset if necessary
-        ret = defaultCharset();
+        ret = Content::defaultCharset();
     }
     return ret;
 }
 
 void ContentType::setCharset(const QByteArray & s) {
-    setParameter(QLatin1String("charset"), QString::fromLatin1(s));
+    setParameter(QStringLiteral("charset"), QString::fromLatin1(s));
 }
 
 QByteArray ContentType::boundary() const {
-    return parameter(QLatin1String("boundary")).toLatin1();
+    return parameter(QStringLiteral("boundary")).toLatin1();
 }
 
 void ContentType::setBoundary(const QByteArray & s) {
-    setParameter(QLatin1String("boundary"), QString::fromLatin1(s));
+    setParameter(QStringLiteral("boundary"), QString::fromLatin1(s));
 }
 
 QString ContentType::name() const {
-    return parameter(QLatin1String("name"));
+    return parameter(QStringLiteral("name"));
 }
 
 void ContentType::setName(const QString & s, const QByteArray & cs) {
     Q_D(ContentType);
     d->encCS = cs;
-    setParameter(QLatin1String("name"), s);
+    setParameter(QStringLiteral("name"), s);
 }
 
 QByteArray ContentType::id() const {
-    return parameter(QLatin1String("id")).toLatin1();
+    return parameter(QStringLiteral("id")).toLatin1();
 }
 
 void ContentType::setId(const QByteArray & s) {
-    setParameter(QLatin1String("id"), QString::fromLatin1(s));
+    setParameter(QStringLiteral("id"), QString::fromLatin1(s));
 }
 
 int ContentType::partialNumber() const {
-    QByteArray p = parameter(QLatin1String("number")).toLatin1();
+    QByteArray p = parameter(QStringLiteral("number")).toLatin1();
     if (!p.isEmpty()) {
         return p.toInt();
     } else {
@@ -1843,7 +1768,7 @@ int ContentType::partialNumber() const {
 }
 
 int ContentType::partialCount() const {
-    QByteArray p = parameter(QLatin1String("total")).toLatin1();
+    QByteArray p = parameter(QStringLiteral("total")).toLatin1();
     if (!p.isEmpty()) {
         return p.toInt();
     } else {
@@ -1861,8 +1786,8 @@ void ContentType::setCategory(contentCategory c) {
 }
 
 void ContentType::setPartialParams(int total, int number) {
-    setParameter(QLatin1String("number"), QString::number(number));
-    setParameter(QLatin1String("total"), QString::number(total));
+    setParameter(QStringLiteral("number"), QString::number(number));
+    setParameter(QStringLiteral("total"), QString::number(total));
 }
 
 bool ContentType::parse(const char *&scursor, const char *const send,
@@ -2135,11 +2060,11 @@ void ContentDisposition::setDisposition(contentDisposition disp) {
 }
 
 QString KMime::Headers::ContentDisposition::filename() const {
-    return parameter(QLatin1String("filename"));
+    return parameter(QStringLiteral("filename"));
 }
 
 void ContentDisposition::setFilename(const QString & filename) {
-    setParameter(QLatin1String("filename"), filename);
+    setParameter(QStringLiteral("filename"), filename);
 }
 
 bool ContentDisposition::parse(const char  *&scursor, const char *const send,
@@ -2188,12 +2113,8 @@ bool ContentDisposition::parse(const char  *&scursor, const char *const send,
 kmime_mk_trivial_ctor_with_name(Subject, Generics::Unstructured, Subject)
 //@endcond
 
-bool Subject::isReply() const {
-    return asUnicodeString().indexOf(QLatin1String("Re:"), 0, Qt::CaseInsensitive) == 0;
-}
-
 Base *createHeader(const QByteArray & type) {
-    return HeaderFactory::self()->createHeader(type);
+    return HeaderFactory::createHeader(type);
 }
 
 //@cond PRIVATE
